/*	Praca domowa ZPR
Katarzyna Rusinowska
Grupa studencka:	N1-IBERE2
Temat pracy:	boost::asio (obsuga timerw)
*/

/*
Operacje wejcia/wyjcia czsto maj wyznaczony czas w przecigu, ktrego musz zosta wykonane. 
Czas ten moe by okrelony jako czas absolutny, bd obliczony w stosunku do czasu biecego.
W celu sprecyzowania owego czasu z pomoc przychodzi nam biblioteka Asio wraz z klas deadline_timer,
ktra pozwala nam na zaimplementowanie timerw synchronicznych i asynchronicznych.
*/

/*
Waniejsze funkcje dostpne w klasie deadline_timer :
-basic_deadline_timer::async_wait
async_wait(handler) suy do inicjacji asynchronicznego czekania na timer, kade wywoanie async_wait 
spowoduje wywoanie dokadnie raz handlera, w przypadku kiedy: 
a)minie czas 
b)dziaanie zegara zostanie anulowane(zostanie wywoany kod bdu: boost::asio::error::operation_aborted)  
Wywoywany handler musi mie nastpujca form:
void handler(const boost::system::error_code& error / Result of operation/);

-basic_deadline_timer::expires_from_now
ustawienie czasu upynicia terminu w stosunku do czasu biecego np. expires_from_now(boost::posix_time::seconds(5))

-basic_deadline_timer::expires_at0
ustawienie terminu ostatecznego wyganicia jako czasu absolutnego 

-basic_deadline_timer::cancel
anuluje kad asynchroniczn operacj, na ktr czeka timer

-basic_deadline_timer::wait 
wykonuje "blokujce" czekanie na timer
*/

/*
Zaimplementowane konstruktory umoliwiaj:
-tworzenie timera bez ustawiania czasu wyganicia, np. boost::asio::deadline_timer t(io), wymaga wywoania po niej funkcji expires_at() 
albo expires_from_now()aby skonfigurowa czas wyganicia

-tworzenie timera z absolutnym czasem wyganicia, 

-tworzenie timera z aktualnym czasem wyganicia, 
np. boost::asio::deadline_timer t(io,boost::posix_time::time_from_string("2009-05-01 23:59:59.000"))
*/

/*
Oglny algorytm wykorzystania timerw przy operacjach asynchronicznych(tu asynchroniczne czytanie z socketa):
 1)io_service ios;
  deadline_timer t(ios);
  tcp::socket socket(ios);
 2)socket.async_read()
 3)timer_.expires_from_now(boost::posix_time::seconds(5));
   timer_.async_wait(handler);
W przypadku przerwania odczytu z soceta na duej ni 5 sekund zostanie wywoany handler.
*/

//PRZYKAD OBSUGI TIMERW Z WYKORZYSTANIEM BIBLIOTEKI boost::asio
#include <iostream>
#include <boost/asio.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/bind.hpp>


//Operacja synchronicznego czekania z wykorzystaniem czasu aktualnego, blokuje dalsze wykonywane zadania
void timer_synch()
{

	boost::asio::io_service io;//niezbdna usuga potrzebna kademu obiektowi boost::asio do przeprowadzenia operacji I/O 
							   //uatwia wykonywanie operacji na asynchronicznych funkcjach

	boost::asio::deadline_timer t(io);

	t.expires_from_now(boost::posix_time::seconds(10));//ustawienie czasu upynicia terminu w stosunku do czasu biecego
													   //lub boost::asio::deadline_timer t(io,boost::posix_time::seconds(5)); 
	std::cout<<"Testowanie timera synchronicznego rozpoczete\n";
	t.wait();//gwarantuje brak powrotu dopki zegar nie wyganie
	std::cout<<"Testowanie timera synchronicznego zakonczone\n";

}

//Funkcja wywoywana w przypadku wyganicia czasu oczekiwania timera asynchronicznego
void handler(const boost::system::error_code& e)
{
	std::cout << "Handler timer asynchronicznego: Czas minal\n";
}

//Operacja asynchronicznego czekania z wykorzystaniem czasu aktualnego
void timer_asynch()
{

	boost::asio::io_service io;//niezbdna usuga potrzebna kademu obiektowi boost::asio do przeprowadzenia operacji I/O 
							   //uatwia wykonywanie operacji na asynchronicznych funkcjach

	boost::asio::deadline_timer t(io, boost::posix_time::seconds(10));
	
	std::cout<<"Testowanie timera asynchronicznego rozpoczete\n";
	t.async_wait(handler);//inicjacja asynchronicznego czekania na timer
	std::cout<<"Wtrakcie czekania ...\n";
	io.run();//biblioteka asio gwarantuje nam, e callback  handlera bdzie tylko wywoany z wtku , ktry obecnie wywouje io_service::run()
	std::cout<<"Testowanie timera asynchronicznego zakonczone\n";
}


int main()
{
	try{
		timer_synch();
		timer_asynch();
	}
	catch (std::exception& e)
	{
		std::cout << "Wyjatek: " << e.what() << "\n";
		getchar();
	}
	return 0;
}